/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.api.type.component;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import dev.onyxstudios.cca.api.v3.component.sync.AutoSyncedComponent;
import java.util.List;
import nerdhub.cardinal.components.api.component.Component;
import nerdhub.cardinal.components.api.component.extension.CopyableComponent;
import net.minecraft.class_1309;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1799;
import net.minecraft.class_2392;
import net.minecraft.class_2398;
import net.minecraft.class_243;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import org.jetbrains.annotations.ApiStatus;
import top.theillusivec4.curios.api.type.ISlotType;

public interface ICurio extends AutoSyncedComponent, CopyableComponent<ICurio> {

  /*
   * Copy of vanilla implementation for breaking items client-side
   */
  static void playBreakAnimation(class_1799 stack, class_1309 livingEntity) {

    if (!stack.method_7960()) {

      if (!livingEntity.method_5701()) {
        livingEntity.field_6002.method_8486(livingEntity.method_23317(), livingEntity.method_23318(), livingEntity.method_23321(),
            class_3417.field_15075, livingEntity.method_5634(), 0.8F,
            0.8F + livingEntity.field_6002.field_9229.nextFloat() * 0.4F, false);
      }

      for (int i = 0; i < 5; ++i) {
        class_243 vec3d = new class_243(((double) livingEntity.method_6051().nextFloat() - 0.5D) * 0.1D,
            Math.random() * 0.1D + 0.1D, 0.0D);
        vec3d = vec3d.method_1037(-livingEntity.field_5965 * ((float) Math.PI / 180F));
        vec3d = vec3d.method_1024(-livingEntity.field_6031 * ((float) Math.PI / 180F));
        double d0 = (double) (-livingEntity.method_6051().nextFloat()) * 0.6D - 0.3D;

        class_243 vec3d1 = new class_243(((double) livingEntity.method_6051().nextFloat() - 0.5D) * 0.3D, d0,
            0.6D);
        vec3d1 = vec3d1.method_1037(-livingEntity.field_5965 * ((float) Math.PI / 180F));
        vec3d1 = vec3d1.method_1024(-livingEntity.field_6031 * ((float) Math.PI / 180F));
        vec3d1 = vec3d1.method_1031(livingEntity.method_23317(), livingEntity.method_23318() + livingEntity.method_23320(),
            livingEntity.method_23321());

        livingEntity.field_6002
            .method_8406(new class_2392(class_2398.field_11218, stack), vec3d1.field_1352, vec3d1.field_1351,
                vec3d1.field_1350, vec3d.field_1352, vec3d.field_1351 + 0.05D, vec3d.field_1350);
      }
    }
  }

  /**
   * Called every tick on both client and server while the ItemStack is equipped.
   *
   * @param identifier   The {@link ISlotType} identifier of the ItemStack's slot
   * @param index        The index of the slot
   * @param livingEntity The wearer of the ItemStack
   */
  default void curioTick(String identifier, int index, class_1309 livingEntity) {

  }

  /**
   * Called every tick only on the client while the ItemStack is equipped.
   *
   * @param identifier   The {@link ISlotType} identifier of the ItemStack's slot
   * @param index        The index of the slot
   * @param livingEntity The wearer of the ItemStack
   */
  default void curioAnimate(String identifier, int index, class_1309 livingEntity) {

  }

  /**
   * Called when the ItemStack is equipped into a slot.
   *
   * @param identifier   The {@link ISlotType} identifier of the slot being equipped into
   * @param index        The index of the slot
   * @param livingEntity The wearer of the ItemStack
   */
  default void onEquip(String identifier, int index, class_1309 livingEntity) {

  }

  /**
   * Called when the ItemStack is unequipped from a slot.
   *
   * @param identifier   The {@link ISlotType} identifier of the slot being unequipped from
   * @param index        The index of the slot
   * @param livingEntity The wearer of the ItemStack
   */
  default void onUnequip(String identifier, int index, class_1309 livingEntity) {

  }

  /**
   * Determines if the ItemStack can be equipped into a slot.
   *
   * @param identifier   The {@link ISlotType} identifier of the slot being equipped into
   * @param livingEntity The wearer of the ItemStack
   * @return True if the ItemStack can be equipped/put in, false if not
   */
  default boolean canEquip(String identifier, class_1309 livingEntity) {
    return true;
  }

  /**
   * Determines if the ItemStack can be unequipped from a slot.
   *
   * @param identifier   The {@link ISlotType} identifier of the slot being unequipped from
   * @param livingEntity The wearer of the ItemStack
   * @return True if the ItemStack can be unequipped/taken out, false if not
   */
  default boolean canUnequip(String identifier, class_1309 livingEntity) {
    return true;
  }

  /**
   * Retrieves a list of tooltips when displaying curio tag information. By default, this will be a
   * list of each tag identifier, translated and in gold text, associated with the curio.
   * <br>
   * If overriding, make sure the user has some indication which tags are associated with the
   * curio.
   *
   * @param tagTooltips A list of {@link Text} with every curio tag
   * @return A list of ITextComponent to display as curio tag information
   */
  default List<class_2561> getTagsTooltip(List<class_2561> tagTooltips) {
    return tagTooltips;
  }

  /**
   * A map of AttributeModifier associated with the ItemStack and the {@link ISlotType} identifier.
   *
   * @param identifier The CurioType identifier for the context
   * @return A map of attribute modifiers to apply
   */
  default Multimap<class_1320, class_1322> getAttributeModifiers(
      String identifier) {
    return HashMultimap.create();
  }

  /**
   * Plays a sound server-side when a curio is equipped from right-clicking the ItemStack in hand.
   * This can be overridden to play nothing, but it is advised to always play something as an
   * auditory feedback for players.
   *
   * @param livingEntity The wearer of the ItemStack
   */
  default void playRightClickEquipSound(class_1309 livingEntity) {
    livingEntity.field_6002
        .method_8396(null, livingEntity.method_24515(), class_3417.field_14883,
            class_3419.field_15254, 1.0f, 1.0f);
  }

  /**
   * Determines if the ItemStack can be automatically equipped into the first available slot when
   * right-clicked.
   *
   * @return True to enable right-clicking auto-equip, false to disable
   */
  default boolean canRightClickEquip() {
    return false;
  }

  /**
   * Called when rendering break animations and sounds client-side when a worn curio item is
   * broken.
   *
   * @param stack        The ItemStack that was broken
   * @param livingEntity The entity that broke the curio
   */
  default void curioBreak(class_1799 stack, class_1309 livingEntity) {
    playBreakAnimation(stack, livingEntity);
  }

  /**
   * Determines if the ItemStack should drop on death and persist through respawn. This will persist
   * the ItemStack in the curio slot to the respawned player if applicable.
   *
   * @param livingEntity The entity that died
   * @return {@link DropRule}
   */
  default DropRule getDropRule(class_1309 livingEntity) {
    return DropRule.DEFAULT;
  }

  /**
   * Used by {@link ICurio#getDropRule(LivingEntity)} to determine drop on death behavior.
   * <br>
   * DEFAULT - normal vanilla behavior with drops dictated by the Keep Inventory game rule
   * <br>
   * ALWAYS_DROP - always drop regardless of game rules
   * <br>
   * ALWAYS_KEEP - always keep regardless of game rules
   * <br>
   * DESTROY - destroy the item upon death
   */
  enum DropRule {
    DEFAULT, ALWAYS_DROP, ALWAYS_KEEP, DESTROY
  }

  @Override
  default boolean isComponentEqual(Component other) {
    return true;
  }

  @Override
  default void readFromNbt(class_2487 compoundTag) {
    this.fromTag(compoundTag);
  }

  @Override
  default void writeToNbt(class_2487 compoundTag) {
    this.toTag(compoundTag);
  }

  @Deprecated
  @ApiStatus.ScheduledForRemoval
  default void fromTag(class_2487 var1) {

  }

  @Deprecated
  @ApiStatus.ScheduledForRemoval
  default class_2487 toTag(class_2487 var1) {
    return new class_2487();
  }
}
