/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.client;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import net.fabricmc.fabric.api.network.ClientSidePacketRegistry;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1703;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_437;
import net.minecraft.class_746;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.type.component.ICurio;
import top.theillusivec4.curios.client.screen.CuriosScreen;
import top.theillusivec4.curios.common.CuriosNetwork;
import top.theillusivec4.curios.common.inventory.screen.CuriosScreenHandler;
import top.theillusivec4.curios.server.CurioArgumentType;

public class CuriosClientNetwork {

  public static void registerPackets() {
    ClientSidePacketRegistry.INSTANCE
        .register(CuriosNetwork.BREAK, (((packetContext, packetByteBuf) -> {
          int entityId = packetByteBuf.readInt();
          String curioId = packetByteBuf.method_10800(25);
          int index = packetByteBuf.readInt();

          packetContext.getTaskQueue().execute(() -> {
            class_310 client = class_310.method_1551();

            if (client != null && client.field_1687 != null) {
              class_1297 entity = client.field_1687.method_8469(entityId);

              if (entity instanceof class_1309) {
                class_1309 livingEntity = (class_1309) entity;
                CuriosApi.getCuriosHelper().getCuriosHandler(livingEntity).ifPresent(handler -> {
                  class_1799 stack = handler.getStacksHandler(curioId)
                      .map(stacksHandler -> stacksHandler.getStacks().method_5438(index))
                      .orElse(class_1799.field_8037);
                  Optional<ICurio> maybeCurio = CuriosApi.getCuriosHelper().getCurio(stack);
                  maybeCurio.ifPresent(curio -> curio.curioBreak(stack, livingEntity));

                  if (!maybeCurio.isPresent()) {
                    ICurio.playBreakAnimation(stack, livingEntity);
                  }
                });
              }
            }
          });
        })));

    ClientSidePacketRegistry.INSTANCE
        .register(CuriosNetwork.SET_ICONS, (((packetContext, packetByteBuf) -> {
          int entrySize = packetByteBuf.readInt();
          Map<String, class_2960> map = new HashMap<>();

          for (int i = 0; i < entrySize; i++) {
            map.put(packetByteBuf.method_10800(25), new class_2960(packetByteBuf.method_10800(100)));
          }
          packetContext.getTaskQueue().execute(() -> {
            Set<String> slotIds = new HashSet<>();
            CuriosApi.getIconHelper().clearIcons();

            for (Map.Entry<String, class_2960> entry : map.entrySet()) {
              CuriosApi.getIconHelper().addIcon(entry.getKey(), entry.getValue());
              slotIds.add(entry.getKey());
            }
            CurioArgumentType.slotIds = slotIds;
          });
        })));

    ClientSidePacketRegistry.INSTANCE
        .register(CuriosNetwork.GRAB_ITEM, (((packetContext, packetByteBuf) -> {
          class_1799 stack = packetByteBuf.method_10819();
          packetContext.getTaskQueue().execute(() -> {
            class_310 client = class_310.method_1551();
            class_746 clientPlayerEntity = client.field_1724;

            if (clientPlayerEntity != null) {
              clientPlayerEntity.field_7514.method_7396(stack);
            }
          });
        })));

    ClientSidePacketRegistry.INSTANCE
        .register(CuriosNetwork.SCROLL, (((packetContext, packetByteBuf) -> {
          int syncId = packetByteBuf.readInt();
          int scrollIndex = packetByteBuf.readInt();

          packetContext.getTaskQueue().execute(() -> {
            class_310 client = class_310.method_1551();
            class_746 clientPlayerEntity = client.field_1724;
            class_437 screen = client.field_1755;

            if (clientPlayerEntity != null) {
              class_1703 screenHandler = clientPlayerEntity.field_7512;

              if (screenHandler instanceof CuriosScreenHandler && screenHandler.field_7763 == syncId) {
                ((CuriosScreenHandler) screenHandler).scrollToIndex(scrollIndex);
              }
            }

            if (screen instanceof CuriosScreen) {
              ((CuriosScreen) screen).updateRenderButtons();
            }
          });
        })));
  }
}
