/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.client.screen;

import com.mojang.blaze3d.systems.RenderSystem;
import io.netty.buffer.Unpooled;
import net.fabricmc.fabric.api.network.ClientSidePacketRegistry;
import net.minecraft.class_1661;
import net.minecraft.class_1713;
import net.minecraft.class_1735;
import net.minecraft.class_2540;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_339;
import net.minecraft.class_344;
import net.minecraft.class_3532;
import net.minecraft.class_3545;
import net.minecraft.class_4587;
import net.minecraft.class_465;
import net.minecraft.class_490;
import net.minecraft.class_507;
import net.minecraft.class_518;
import net.minecraft.class_746;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.type.component.ICuriosItemHandler;
import top.theillusivec4.curios.client.CuriosClientConfig.ButtonCorner;
import top.theillusivec4.curios.client.KeyRegistry;
import top.theillusivec4.curios.common.CuriosNetwork;
import top.theillusivec4.curios.common.inventory.CosmeticCurioSlot;
import top.theillusivec4.curios.common.inventory.CurioSlot;
import top.theillusivec4.curios.common.inventory.screen.CuriosScreenHandler;

public class CuriosScreen extends class_465<CuriosScreenHandler> implements class_518 {

  public static final class_2960 CURIO_INVENTORY = new class_2960(CuriosApi.MODID,
      "textures/gui/inventory.png");
  public static final class_2960 RECIPE_BUTTON_TEXTURE = new class_2960(
      "minecraft:textures/gui/recipe_button.png");
  private static final class_2960 CREATIVE_INVENTORY_TABS = new class_2960(
      "textures/gui/container/creative_inventory/tabs.png");

  private static float currentScroll;

  private final class_507 recipeBook = new class_507();

  public boolean hasScrollBar;
  public boolean isNarrow;
  public float currentMouseX;
  public float currentMouseY;

  private CuriosButton buttonCurios;
  private boolean isScrolling;
  private boolean buttonClicked;
  private boolean isRenderButtonHovered;

  public CuriosScreen(CuriosScreenHandler handler, class_1661 inventory, class_2561 title) {
    super(handler, inventory, title);
    this.field_22792 = true;
    this.field_25267 = 97;
  }

  public static class_3545<Integer, Integer> getButtonOffset(boolean isCreative) {
    ButtonCorner corner = ButtonCorner.TOP_LEFT;
    int left = 0;
    int right = 0;

    if (isCreative) {
      left += corner.getCreativeXoffset();
      right += corner.getCreativeYoffset();
    } else {
      left += corner.getXoffset();
      right += corner.getYoffset();
    }
    return new class_3545<>(left, right);
  }

  @Override
  protected void method_25426() {
    super.method_25426();

    if (this.field_22787 != null) {

      if (this.field_22787.field_1724 != null) {
        hasScrollBar = CuriosApi.getCuriosHelper().getCuriosHandler(this.field_22787.field_1724)
            .map(handler -> handler.getVisibleSlots() > 8).orElse(false);

        if (hasScrollBar) {
          this.method_17577().scrollToPosition(currentScroll);
        }
      }

      int neededWidth = 431;

      if (hasScrollBar) {
        neededWidth += 30;
      }

      if (field_2797.hasCosmeticColumn()) {
        neededWidth += 40;
      }
      this.isNarrow = this.field_22789 < neededWidth;
      this.recipeBook.method_2597(this.field_22789, this.field_22790, this.field_22787, this.isNarrow, this.field_2797);
      this.updateScreenPosition();
      this.field_22786.add(this.recipeBook);
      this.method_20085(this.recipeBook);
      class_3545<Integer, Integer> offsets = getButtonOffset(false);
      this.buttonCurios = new CuriosButton(this, this.field_2776 + offsets.method_15442(),
          this.field_22790 / 2 + offsets.method_15441(), 14, 14, 50, 0, 14, CURIO_INVENTORY);
      this.method_25411(this.buttonCurios);

      if (!this.field_17410.field_7546.method_7337()) {

        this.method_25411(
            new class_344(this.field_2776 + 104, this.field_22790 / 2 - 22, 20, 18, 0, 0, 19,
                RECIPE_BUTTON_TEXTURE, (buttonWidget) -> {
              this.recipeBook.method_2579(this.isNarrow);
              this.recipeBook.method_2591();
              this.updateScreenPosition();
              ((class_344) buttonWidget).method_1893(this.field_2776 + 104, this.field_22790 / 2 - 22);
              this.buttonCurios
                  .method_1893(this.field_2776 + offsets.method_15442(), this.field_22790 / 2 + offsets.method_15441());
            }));
      }
      this.updateRenderButtons();
    }
  }

  public void updateRenderButtons() {
    this.field_22791.removeIf(widget -> widget instanceof RenderToggleButton);
    this.field_22786.removeIf(widget -> widget instanceof RenderToggleButton);
    int yOffset = 9;

    for (class_1735 inventorySlot : this.method_17577().field_7761) {

      if (inventorySlot instanceof CurioSlot && !(inventorySlot instanceof CosmeticCurioSlot)) {
        this.method_25411(
            new RenderToggleButton((CurioSlot) inventorySlot, this.field_2776 - 8, this.field_2800 + yOffset, 8, 8,
                75, 0, 8, CURIO_INVENTORY, (button) -> {
              int index = ((CurioSlot) inventorySlot).getIndex();
              String id = ((CurioSlot) inventorySlot).getIdentifier();
              class_2540 packetByteBuf = new class_2540(Unpooled.buffer());
              packetByteBuf.writeInt(index);
              packetByteBuf.method_10788(id, 25);
              ((CurioSlot) inventorySlot).toggleRenderStatus();
              ClientSidePacketRegistry.INSTANCE
                  .sendToServer(CuriosNetwork.TOGGLE_RENDER, packetByteBuf);
            }));
        yOffset += 18;
      }
    }
  }

  private void updateScreenPosition() {
    int i;

    if (this.recipeBook.method_2605() && !this.isNarrow) {
      int offset = 148;

      if (hasScrollBar) {
        offset -= 30;
      }

      if (field_2797.hasCosmeticColumn()) {
        offset -= 40;
      }
      i = 177 + (this.field_22789 - this.field_2792 - offset) / 2;
    } else {
      i = (this.field_22789 - this.field_2792) / 2;
    }
    this.field_2776 = i;
    this.updateRenderButtons();
  }

  @Override
  public void method_25393() {
    super.method_25393();
    this.recipeBook.method_2590();
  }

  private boolean inScrollBar(double mouseX, double mouseY) {
    int i = this.field_2776;
    int j = this.field_2800;
    int k = i - 34;
    int l = j + 12;
    int i1 = k + 14;
    int j1 = l + 139;

    if (this.method_17577().hasCosmeticColumn()) {
      i1 -= 19;
      k -= 19;
    }
    return mouseX >= (double) k && mouseY >= (double) l && mouseX < (double) i1
        && mouseY < (double) j1;
  }

  @Override
  public void method_25394(class_4587 matrices, int mouseX, int mouseY, float delta) {
    this.method_25420(matrices);

    if (this.recipeBook.method_2605() && this.isNarrow) {
      this.method_2389(matrices, delta, mouseX, mouseY);
      this.recipeBook.method_25394(matrices, mouseX, mouseY, delta);
    } else {
      this.recipeBook.method_25394(matrices, mouseX, mouseY, delta);
      super.method_25394(matrices, mouseX, mouseY, delta);
      this.recipeBook.method_2581(matrices, this.field_2776, this.field_2800, false, delta);

      boolean isButtonHovered = false;

      for (class_339 button : this.field_22791) {

        if (button instanceof RenderToggleButton) {
          ((RenderToggleButton) button).renderButtonOverlay(matrices, mouseX, mouseY, delta);

          if (button.method_25367()) {
            isButtonHovered = true;
            break;
          }
        }
      }
      this.isRenderButtonHovered = isButtonHovered;
      class_746 clientPlayerEntity = class_310.method_1551().field_1724;

      if (!this.isRenderButtonHovered && clientPlayerEntity != null && clientPlayerEntity.field_7514
          .method_7399().method_7960() && this.field_2787 != null) {
        class_1735 slot = this.field_2787;

        if (slot instanceof CurioSlot && !slot.method_7681()) {
          this.method_25424(matrices, new class_2585(((CurioSlot) slot).getSlotName()), mouseX,
              mouseY);
        }
      }
    }
    this.method_2380(matrices, mouseX, mouseY);
    currentMouseX = mouseX;
    currentMouseY = mouseY;
  }

  @Override
  protected void method_2380(class_4587 matrices, int x, int y) {

    if (this.field_22787 != null && this.field_22787.field_1724 != null && this.field_22787.field_1724.field_7514
        .method_7399().method_7960()) {

      if (this.isRenderButtonHovered) {
        this.method_25424(matrices, new class_2588("gui.curios.toggle"), x, y);
      } else if (this.field_2787 != null && this.field_2787.method_7681()) {
        this.method_25409(matrices, this.field_2787.method_7677(), x, y);
      }
    }
  }

  @Override
  public boolean method_25404(int keyCode, int scanCode, int modifiers) {
    if (this.recipeBook.method_2605() && this.isNarrow) {
      this.recipeBook.method_2591();
      this.updateScreenPosition();
      return true;
    } else if (KeyRegistry.openCurios.method_1417(keyCode, scanCode)) {

      if (this.field_22787 != null && this.field_22787.field_1724 != null) {
        this.field_22787.field_1724.method_7346();
      }
      return true;
    } else {
      return super.method_25404(keyCode, scanCode, modifiers);
    }
  }

  @SuppressWarnings("deprecation")
  @Override
  protected void method_2389(class_4587 matrices, float delta, int mouseX, int mouseY) {

    if (this.field_22787 != null && this.field_22787.field_1724 != null) {
      RenderSystem.color4f(1.0F, 1.0F, 1.0F, 1.0F);
      this.field_22787.method_1531().method_22813(field_2801);
      int i = this.field_2776;
      int j = this.field_2800;
      this.method_25302(matrices, i, j, 0, 0, this.field_2792, this.field_2779);
      class_490
          .method_2486(i + 51, j + 75, 30, (float) (i + 51) - mouseX, (float) (j + 75 - 50) - mouseY,
              this.field_22787.field_1724);
      CuriosApi.getCuriosHelper().getCuriosHandler(this.field_22787.field_1724).ifPresent(handler -> {
        int slotCount = handler.getVisibleSlots();

        if (slotCount > 0) {
          int upperHeight = 7 + slotCount * 18;
          RenderSystem.color4f(1.0f, 1.0f, 1.0f, 1.0f);
          this.field_22787.method_1531().method_22813(CURIO_INVENTORY);
          int xTexOffset = 0;
          int width = 27;
          int xOffset = -26;

          if (this.method_17577().hasCosmeticColumn()) {
            xTexOffset = 92;
            width = 46;
            xOffset -= 19;
          }
          this.method_25302(matrices, i + xOffset, j + 4, xTexOffset, 0, width, upperHeight);

          if (slotCount <= 8) {
            this.method_25302(matrices, i + xOffset, j + 4 + upperHeight, xTexOffset, 151, width, 7);
          } else {
            this.method_25302(matrices, i + xOffset - 16, j + 4, 27, 0, 23, 158);
            this.field_22787.method_1531().method_22813(CREATIVE_INVENTORY_TABS);
            this.method_25302(matrices, i + xOffset - 8, j + 12 + (int) (127f * currentScroll), 232,
                0, 12, 15);
          }

          for (class_1735 slot : this.method_17577().field_7761) {

            if (slot instanceof CosmeticCurioSlot) {
              int x = this.field_2776 + slot.field_7873 - 1;
              int y = this.field_2800 + slot.field_7872 - 1;
              this.field_22787.method_1531().method_22813(CURIO_INVENTORY);
              this.method_25302(matrices, x, y, 138, 0, 18, 18);
            }
          }
        }
      });
    }
  }

  @Override
  protected void method_2388(class_4587 matrices, int mouseX, int mouseY) {
    this.field_22793.method_30883(matrices, this.field_22785, (float) this.field_25267, (float) this.field_25268, 4210752);
  }

  @Override
  protected boolean method_2378(int xPosition, int yPosition, int width, int height,
      double pointX, double pointY) {

    if (this.isRenderButtonHovered) {
      return false;
    }
    return (!this.isNarrow || !this.recipeBook.method_2605()) && super
        .method_2378(xPosition, yPosition, width, height, pointX, pointY);
  }

  @Override
  public boolean method_25402(double mouseX, double mouseY, int button) {
    if (this.recipeBook.method_25402(mouseX, mouseY, button)) {
      return true;
    } else if (this.inScrollBar(mouseX, mouseY)) {
      this.isScrolling = this.needsScrollBars();
      return true;
    }
    return this.isNarrow && this.recipeBook.method_2605() || super.method_25402(mouseX, mouseY, button);
  }

  @Override
  public boolean method_25406(double mouseX, double mouseY, int button) {
    if (button == 0) {
      this.isScrolling = false;
    }

    if (this.buttonClicked) {
      this.buttonClicked = false;
      return true;
    } else {
      return super.method_25406(mouseX, mouseY, button);
    }
  }

  @Override
  public boolean method_25403(double mouseX, double mouseY, int button, double deltaX,
      double deltaY) {

    if (this.isScrolling) {
      int i = this.field_2800 + 8;
      int j = i + 148;
      currentScroll = ((float) mouseY - (float) i - 7.5F) / ((float) (j - i) - 15.0F);
      currentScroll = class_3532.method_15363(currentScroll, 0.0F, 1.0F);
      this.method_17577().scrollToPosition(currentScroll);
      return true;
    } else {
      return super.method_25403(mouseX, mouseY, button, deltaX, deltaY);
    }
  }

  @Override
  public boolean method_25401(double mouseX, double mouseY, double amount) {

    if (!this.needsScrollBars()) {
      return false;
    } else {
      int i = this.method_17577().getCuriosHandler().map(ICuriosItemHandler::getVisibleSlots)
          .orElse(1);
      currentScroll = (float) ((double) currentScroll - amount / (double) i);
      currentScroll = class_3532.method_15363(currentScroll, 0.0F, 1.0F);
      this.method_17577().scrollToPosition(currentScroll);
      return true;
    }
  }

  private boolean needsScrollBars() {
    return this.method_17577().canScroll();
  }

  @Override
  protected boolean method_2381(double mouseX, double mouseY, int left, int top,
      int button) {
    int offset = -20;

    if (hasScrollBar) {
      offset -= 20;
    }

    if (this.method_17577().hasCosmeticColumn()) {
      offset -= 20;
    }
    boolean bl =
        mouseX < (double) (left + offset) || mouseY < (double) top || mouseX >= (double) (left
            + this.field_2792) || mouseY >= (double) (top + this.field_2779);
    return this.recipeBook
        .method_2598(mouseX, mouseY, this.field_2776, this.field_2800, this.field_2792,
            this.field_2779, button) && bl;
  }

  @Override
  protected void method_2383(class_1735 slot, int invSlot, int clickData, class_1713 actionType) {
    super.method_2383(slot, invSlot, clickData, actionType);
    this.recipeBook.method_2600(slot);
  }

  @Override
  public void method_16891() {
    this.recipeBook.method_2592();
  }

  @Override
  public void method_25432() {
    this.recipeBook.method_2607();
    super.method_25432();
  }

  @Override
  public class_507 method_2659() {
    return this.recipeBook;
  }
}
