/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.common.inventory;

import com.mojang.datafixers.util.Pair;
import java.util.Set;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1074;
import net.minecraft.class_1657;
import net.minecraft.class_1723;
import net.minecraft.class_1735;
import net.minecraft.class_1799;
import net.minecraft.class_1890;
import net.minecraft.class_2371;
import net.minecraft.class_2960;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.type.inventory.IDynamicStackHandler;

public class CurioSlot extends class_1735 {

  protected final String identifier;
  protected final class_1657 player;
  protected final int index;

  private class_2371<Boolean> renderStatuses;

  public CurioSlot(class_1657 player, IDynamicStackHandler handler, int index, String identifier,
                   int xPosition, int yPosition, class_2371<Boolean> renders) {
    super(handler, index, xPosition, yPosition);
    this.identifier = identifier;
    this.index = index;
    this.renderStatuses = renders;
    this.player = player;
  }

  @Environment(EnvType.CLIENT)
  @Override
  public Pair<class_2960, class_2960> method_7679() {
    return new Pair<>(class_1723.field_21668,
        player.method_5770().method_8608() ? CuriosApi.getIconHelper().getIcon(identifier)
            : new class_2960(CuriosApi.MODID, "item/empty_curio_slot"));
  }

  public String getIdentifier() {
    return this.identifier;
  }

  public int getIndex() {
    return this.index;
  }

  public boolean getRenderStatus() {
    return this.renderStatuses.get(this.index);
  }

  public void toggleRenderStatus() {
    this.renderStatuses.set(this.index, !getRenderStatus());
  }

  @Environment(EnvType.CLIENT)
  public String getSlotName() {
    return class_1074.method_4662("curios.identifier." + identifier);
  }

  @Override
  public boolean method_7680(class_1799 stack) {
    return hasValidTag(CuriosApi.getCuriosHelper().getCurioTags(stack.method_7909())) && CuriosApi
        .getCuriosHelper().getCurio(stack).map(curio -> curio.canEquip(identifier, player))
        .orElse(true) && super.method_7680(stack);
  }

  protected boolean hasValidTag(Set<String> tags) {
    return tags.contains(identifier) || tags.contains("curio");
  }

  @Override
  public boolean method_7674(class_1657 playerIn) {
    class_1799 stack = this.method_7677();
    return (stack.method_7960() || playerIn.method_7337() || !class_1890.method_8224(stack))
        && CuriosApi.getCuriosHelper().getCurio(stack)
        .map(curio -> curio.canUnequip(identifier, playerIn)).orElse(true) && super
        .method_7674(playerIn);
  }
}
