/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.mixin;

import com.google.common.collect.Multimap;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.UUID;
import net.minecraft.class_124;
import net.minecraft.class_1310;
import net.minecraft.class_1320;
import net.minecraft.class_1322;
import net.minecraft.class_1657;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1836;
import net.minecraft.class_1890;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_5134;
import net.minecraft.class_5250;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.type.component.ICurio;

@Mixin(class_1799.class)
public abstract class ItemStackMixin {

  private static final UUID ATTACK_DAMAGE_MODIFIER_ID = UUID
      .fromString("CB3F55D3-645C-4F38-A497-9C13A33DB5CF");
  private static final UUID ATTACK_SPEED_MODIFIER_ID = UUID
      .fromString("FA233E1C-4180-4865-B01B-BCCE9785ACA3");

  @Shadow
  public abstract class_1792 getItem();

  @Shadow
  public abstract class_2487 getTag();

  @Inject(at = @At("RETURN"), method = "getTooltip", cancellable = true)
  public void getTooltip(class_1657 playerEntity, class_1836 tooltipContext,
      CallbackInfoReturnable<List<class_2561>> cb) {
    List<class_2561> tooltip = cb.getReturnValue();
    @SuppressWarnings("ConstantConditions") class_1799 stack = ((class_1799) (Object) this);
    class_2487 tag = this.getTag();
    int i = 0;

    if (tag != null && tag.method_10573("HideFlags", 99)) {
      i = tag.method_10550("HideFlags");
    }

    Set<String> curioTags = CuriosApi.getCuriosHelper().getCurioTags(this.getItem());
    List<String> slots = new ArrayList<>(curioTags);

    if (!slots.isEmpty()) {
      List<class_2561> tagsTooltip = new ArrayList<>();
      class_5250 slotsTooltip = new class_2588("curios.slot").method_27693(": ")
          .method_27692(class_124.field_1065);

      for (int j = 0; j < slots.size(); j++) {
        String key = "curios.identifier." + slots.get(j);
        class_5250 type = new class_2588(key);

        if (j < slots.size() - 1) {
          type = type.method_27693(", ");
        }
        type = type.method_27692(class_124.field_1054);
        slotsTooltip.method_10852(type);
      }
      tagsTooltip.add(slotsTooltip);
      Optional<ICurio> optionalCurio = CuriosApi.getCuriosHelper().getCurio(stack);
      optionalCurio.ifPresent(curio -> {
        List<class_2561> curioTagsTooltip = curio.getTagsTooltip(tagsTooltip);

        if (!curioTagsTooltip.isEmpty()) {
          tooltip.addAll(1, curio.getTagsTooltip(tagsTooltip));
        }
      });

      if (!optionalCurio.isPresent()) {
        tooltip.addAll(1, tagsTooltip);
      }

      for (String identifier : slots) {
        Multimap<class_1320, class_1322> multimap = CuriosApi.getCuriosHelper()
            .getAttributeModifiers(identifier, stack);

        if (!multimap.isEmpty() && (i & 2) == 0) {
          tooltip.add(class_2585.field_24366);
          tooltip.add(
              new class_2588("curios.modifiers." + identifier).method_27692(class_124.field_1065));

          for (Map.Entry<class_1320, class_1322> entry : multimap.entries()) {
            class_1322 attributeModifier = entry.getValue();
            double amount = attributeModifier.method_6186();
            boolean flag = false;

            if (playerEntity != null) {

              if (attributeModifier.method_6189() == ATTACK_DAMAGE_MODIFIER_ID) {
                amount += playerEntity
                    .method_26826(class_5134.field_23721);
                amount += class_1890.method_8218(stack, class_1310.field_6290);
                flag = true;
              } else if (attributeModifier.method_6189() == ATTACK_SPEED_MODIFIER_ID) {
                amount += playerEntity.method_26826(class_5134.field_23723);
                flag = true;
              }

              double g;

              if (attributeModifier.method_6182()
                  != class_1322.class_1323.field_6330
                  && attributeModifier.method_6182()
                  != class_1322.class_1323.field_6331) {

                if ((entry.getKey()).equals(class_5134.field_23718)) {
                  g = amount * 10.0D;
                } else {
                  g = amount;
                }
              } else {
                g = amount * 100.0D;
              }

              if (flag) {
                tooltip.add((new class_2585(" ")).method_10852(new class_2588(
                    "attribute.modifier.equals." + attributeModifier.method_6182().method_6191(),
                    class_1799.field_8029.format(g),
                    new class_2588((entry.getKey()).method_26830())))
                    .method_27692(class_124.field_1077));
              } else if (amount > 0.0D) {
                tooltip.add((new class_2588(
                    "attribute.modifier.plus." + attributeModifier.method_6182().method_6191(),
                    class_1799.field_8029.format(g),
                    new class_2588((entry.getKey()).method_26830())))
                    .method_27692(class_124.field_1078));
              } else if (amount < 0.0D) {
                g *= -1.0D;
                tooltip.add((new class_2588(
                    "attribute.modifier.take." + attributeModifier.method_6182().method_6191(),
                    class_1799.field_8029.format(g),
                    new class_2588((entry.getKey()).method_26830())))
                    .method_27692(class_124.field_1061));
              }
            }
          }
        }
      }
    }

  }
}
