/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.mixin;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Predicate;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1890;
import net.minecraft.class_1928;
import net.minecraft.class_3545;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.event.CurioChangeCallback;
import top.theillusivec4.curios.api.event.CurioDropsCallback;
import top.theillusivec4.curios.api.event.DropRulesCallback;
import top.theillusivec4.curios.api.type.component.ICurio;
import top.theillusivec4.curios.api.type.component.ICurio.DropRule;
import top.theillusivec4.curios.api.type.inventory.ICurioStacksHandler;
import top.theillusivec4.curios.api.type.inventory.IDynamicStackHandler;

@Mixin(class_1309.class)
public class LivingEntityMixin {

  @Shadow
  int playerHitTimer;

  @Inject(method = "drop", at = @At("TAIL"))
  protected void drop(class_1282 source, CallbackInfo cb) {
    @SuppressWarnings("ConstantConditions") class_1309 livingEntity = (class_1309) (Object) this;
    CuriosApi.getCuriosHelper().getCuriosHandler(livingEntity).ifPresent(handler -> {
      class_1309 wearer = handler.getWearer();
      class_1297 entity = source.method_5529();
      Collection<class_1799> drops = new ArrayList<>();
      int looting =
          entity instanceof class_1309 ? class_1890.method_8226((class_1309) entity) : 0;
      boolean recentlyHit = playerHitTimer > 0;
      List<class_3545<Predicate<class_1799>, DropRule>> overrides = new ArrayList<>();
      DropRulesCallback.EVENT.invoker()
          .dropRules(wearer, handler, source, looting, recentlyHit, overrides);
      boolean keepInventory = wearer.field_6002.method_8450().method_8355(class_1928.field_19389);

      handler.getCurios().forEach((id, stacksHandler) -> {
        handleDrops(wearer, overrides, stacksHandler.getStacks(), drops, keepInventory);
        handleDrops(wearer, overrides, stacksHandler.getCosmeticStacks(), drops, keepInventory);
      });

      if (CurioDropsCallback.EVENT.invoker()
          .drop(livingEntity, handler, source, drops, looting, recentlyHit)) {

        if (wearer instanceof class_1657) {
          drops.forEach(stack -> ((class_1657) wearer).method_7329(stack, true, false));
        } else {
          drops.forEach(livingEntity::method_5775);
        }
      }
    });
  }

  @Inject(method = "tick", at = @At("TAIL"))
  public void tick(CallbackInfo cb) {
    @SuppressWarnings("ConstantConditions") class_1309 livingEntity = (class_1309) (Object) this;
    CuriosApi.getCuriosHelper().getCuriosHandler(livingEntity).ifPresent(handler -> {
      handler.handleInvalidStacks();
      Map<String, ICurioStacksHandler> curios = handler.getCurios();

      for (Map.Entry<String, ICurioStacksHandler> entry : curios.entrySet()) {
        ICurioStacksHandler stacksHandler = entry.getValue();
        String identifier = entry.getKey();
        IDynamicStackHandler stackHandler = stacksHandler.getStacks();
        IDynamicStackHandler cosmeticStackHandler = stacksHandler.getCosmeticStacks();

        for (int i = 0; i < stackHandler.method_5439(); i++) {
          class_1799 stack = stackHandler.method_5438(i);
          Optional<ICurio> currentCurio = CuriosApi.getCuriosHelper().getCurio(stack);
          final int index = i;

          if (!stack.method_7960()) {
            stack.method_7917(livingEntity.field_6002, livingEntity, -1, false);
            currentCurio.ifPresent(curio -> {
              curio.curioTick(identifier, index, livingEntity);

              if (livingEntity.field_6002.method_8608()) {
                curio.curioAnimate(identifier, index, livingEntity);
              }
            });
          }

          if (!livingEntity.field_6002.method_8608()) {
            class_1799 prevStack = stackHandler.getPreviousStack(i);
            boolean sync = false;

            if (!class_1799.method_7973(stack, prevStack)) {
              Optional<ICurio> prevCurio = CuriosApi.getCuriosHelper().getCurio(prevStack);
              sync = true;
              CurioChangeCallback.EVENT.invoker()
                  .change(livingEntity, identifier, i, prevStack, stack);

              if (!prevStack.method_7960()) {
                livingEntity.method_6127().method_26847(
                    CuriosApi.getCuriosHelper().getAttributeModifiers(identifier, prevStack));
                prevCurio.ifPresent(curio -> curio.onUnequip(identifier, index, livingEntity));
              }

              if (!stack.method_7960()) {
                livingEntity.method_6127().method_26854(
                    CuriosApi.getCuriosHelper().getAttributeModifiers(identifier, stack));
                currentCurio.ifPresent(curio -> curio.onEquip(identifier, index, livingEntity));
              }
              stackHandler.setPreviousStack(i, stack.method_7960() ? class_1799.field_8037 : stack.method_7972());
            }
            class_1799 cosmeticStack = cosmeticStackHandler.method_5438(i);
            class_1799 prevCosmeticStack = cosmeticStackHandler.getPreviousStack(i);

            if (!class_1799.method_7973(cosmeticStack, prevCosmeticStack)) {
              sync = true;
              cosmeticStackHandler.setPreviousStack(index,
                  cosmeticStack.method_7960() ? class_1799.field_8037 : cosmeticStack.method_7972());
            }

            if (sync) {
              handler.sync();
            }
          }
        }
      }
    });
  }

  private static void handleDrops(class_1309 livingEntity,
      List<class_3545<Predicate<class_1799>, DropRule>> dropRules, IDynamicStackHandler stacks,
      Collection<class_1799> drops, boolean keepInventory) {

    for (int i = 0; i < stacks.method_5439(); i++) {
      class_1799 stack = stacks.method_5438(i);

      if (!stack.method_7960()) {
        DropRule dropRuleOverride = null;

        for (class_3545<Predicate<class_1799>, DropRule> override : dropRules) {

          if (override.method_15442().test(stack)) {
            dropRuleOverride = override.method_15441();
          }
        }
        DropRule dropRule = dropRuleOverride != null ? dropRuleOverride
            : CuriosApi.getCuriosHelper().getCurio(stack)
                .map(curio -> curio.getDropRule(livingEntity)).orElse(DropRule.DEFAULT);

        if ((dropRule == DropRule.DEFAULT && keepInventory) || dropRule == DropRule.ALWAYS_KEEP) {
          continue;
        }

        if (!class_1890.method_8221(stack) && dropRule != DropRule.DESTROY) {
          drops.add(stack);
        }
        stacks.method_5447(i, class_1799.field_8037);
      }
    }
  }
}
