/*
 * Copyright (c) 2018-2020 C4
 *
 * This file is part of Curios, a mod made for Minecraft.
 *
 * Curios is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Curios is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Curios.  If not, see <https://www.gnu.org/licenses/>.
 */

package top.theillusivec4.curios.server;

import com.mojang.brigadier.Command;
import com.mojang.brigadier.CommandDispatcher;
import com.mojang.brigadier.arguments.IntegerArgumentType;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import java.util.Map;
import net.minecraft.class_1799;
import net.minecraft.class_2168;
import net.minecraft.class_2170;
import net.minecraft.class_2186;
import net.minecraft.class_2588;
import net.minecraft.class_3222;
import top.theillusivec4.curios.api.CuriosApi;
import top.theillusivec4.curios.api.type.inventory.ICurioStacksHandler;

public class CuriosCommand {

  public static void register(CommandDispatcher<class_2168> dispatcher) {

    LiteralArgumentBuilder<class_2168> curiosCommand = class_2170.method_9247("curios")
        .requires(player -> player.method_9259(2));

    curiosCommand.then(class_2170.method_9247("set").then(
        class_2170.method_9244("slot", CurioArgumentType.slot()).then(
            class_2170.method_9244("player", class_2186.method_9305()).executes(
                context -> setSlotsForPlayer(context.getSource(),
                    class_2186.method_9315(context, "player"),
                    CurioArgumentType.getSlot(context, "slot"), 1)).then(
                class_2170.method_9244("amount", IntegerArgumentType.integer()).executes(
                    context -> setSlotsForPlayer(context.getSource(),
                        class_2186.method_9315(context, "player"),
                        CurioArgumentType.getSlot(context, "slot"),
                        IntegerArgumentType.getInteger(context, "amount")))))));

    curiosCommand.then(class_2170.method_9247("add").then(
        class_2170.method_9244("slot", CurioArgumentType.slot()).then(
            class_2170.method_9244("player", class_2186.method_9305()).executes(
                context -> growSlotForPlayer(context.getSource(),
                    class_2186.method_9315(context, "player"),
                    CurioArgumentType.getSlot(context, "slot"), 1)).then(
                class_2170.method_9244("amount", IntegerArgumentType.integer()).executes(
                    context -> growSlotForPlayer(context.getSource(),
                        class_2186.method_9315(context, "player"),
                        CurioArgumentType.getSlot(context, "slot"),
                        IntegerArgumentType.getInteger(context, "amount")))))));

    curiosCommand.then(class_2170.method_9247("remove").then(
        class_2170.method_9244("slot", CurioArgumentType.slot()).then(
            class_2170.method_9244("player", class_2186.method_9305()).executes(
                context -> shrinkSlotForPlayer(context.getSource(),
                    class_2186.method_9315(context, "player"),
                    CurioArgumentType.getSlot(context, "slot"), 1)).then(
                class_2170.method_9244("amount", IntegerArgumentType.integer()).executes(
                    context -> shrinkSlotForPlayer(context.getSource(),
                        class_2186.method_9315(context, "player"),
                        CurioArgumentType.getSlot(context, "slot"),
                        IntegerArgumentType.getInteger(context, "amount")))))));

    curiosCommand.then(class_2170.method_9247("unlock").then(
        class_2170.method_9244("slot", CurioArgumentType.slot()).then(
            class_2170.method_9244("player", class_2186.method_9305()).executes(
                context -> unlockSlotForPlayer(context.getSource(),
                    class_2186.method_9315(context, "player"),
                    CurioArgumentType.getSlot(context, "slot"))))));

    curiosCommand.then(class_2170.method_9247("lock").then(
        class_2170.method_9244("slot", CurioArgumentType.slot()).then(
            class_2170.method_9244("player", class_2186.method_9305()).executes(
                context -> lockSlotForPlayer(context.getSource(),
                    class_2186.method_9315(context, "player"),
                    CurioArgumentType.getSlot(context, "slot"))))));

    curiosCommand.then(class_2170.method_9247("clear").then(
        class_2170.method_9244("player", class_2186.method_9305()).executes(
            context -> clearSlotsForPlayer(context.getSource(),
                class_2186.method_9315(context, "player"), "")).then(
            class_2170.method_9244("slot", CurioArgumentType.slot()).executes(
                context -> clearSlotsForPlayer(context.getSource(),
                    class_2186.method_9315(context, "player"),
                    CurioArgumentType.getSlot(context, "slot"))))));

    curiosCommand.then(class_2170.method_9247("reset").then(
        class_2170.method_9244("player", class_2186.method_9305()).executes(
            context -> resetSlotsForPlayer(context.getSource(),
                class_2186.method_9315(context, "player")))));

    dispatcher.register(curiosCommand);
  }

  private static int setSlotsForPlayer(class_2168 source, class_3222 playerMP,
      String slot, int amount) {
    CuriosApi.getSlotHelper().setSlotsForType(slot, playerMP, amount);
    source.method_9226(new class_2588("commands.curios.set.success", slot,
            CuriosApi.getSlotHelper().getSlotsForType(playerMP, slot), playerMP.method_5476()),
        true);
    return Command.SINGLE_SUCCESS;
  }

  private static int growSlotForPlayer(class_2168 source, class_3222 playerMP,
      String slot, int amount) {
    CuriosApi.getSlotHelper().growSlotType(slot, amount, playerMP);
    source.method_9226(new class_2588("commands.curios.add.success", amount, slot,
        playerMP.method_5476()), true);
    return Command.SINGLE_SUCCESS;
  }

  private static int shrinkSlotForPlayer(class_2168 source, class_3222 playerMP,
      String slot, int amount) {
    CuriosApi.getSlotHelper().shrinkSlotType(slot, amount, playerMP);
    source.method_9226(new class_2588("commands.curios.remove.success", amount, slot,
        playerMP.method_5476()), true);
    return Command.SINGLE_SUCCESS;
  }

  private static int unlockSlotForPlayer(class_2168 source, class_3222 playerMP,
      String slot) {
    CuriosApi.getSlotHelper().unlockSlotType(slot, playerMP);
    source.method_9226(
        new class_2588("commands.curios.unlock.success", slot, playerMP.method_5476()),
        true);
    return Command.SINGLE_SUCCESS;
  }

  private static int lockSlotForPlayer(class_2168 source, class_3222 playerMP,
      String slot) {
    CuriosApi.getSlotHelper().lockSlotType(slot, playerMP);
    source.method_9226(
        new class_2588("commands.curios.lock.success", slot, playerMP.method_5476()),
        true);
    return Command.SINGLE_SUCCESS;
  }

  private static int clearSlotsForPlayer(class_2168 source, class_3222 playerMP,
      String slot) {

    CuriosApi.getCuriosHelper().getCuriosHandler(playerMP).ifPresent(handler -> {
      Map<String, ICurioStacksHandler> curios = handler.getCurios();

      if (!slot.isEmpty() && curios.get(slot) != null) {
        clear(curios.get(slot));
      } else {

        for (String id : curios.keySet()) {
          clear(curios.get(id));
        }
      }
    });

    if (slot.isEmpty()) {
      source.method_9226(
          new class_2588("commands.curios.clearAll.success", playerMP.method_5476()),
          true);
    } else {
      source.method_9226(
          new class_2588("commands.curios.clear.success", slot, playerMP.method_5476()),
          true);
    }
    return Command.SINGLE_SUCCESS;
  }

  private static int resetSlotsForPlayer(class_2168 source, class_3222 playerMP) {
    CuriosApi.getCuriosHelper().getCuriosHandler(playerMP).ifPresent(handler -> {
      handler.reset();
      handler.sync();
    });
    source.method_9226(
        new class_2588("commands.curios.reset.success", playerMP.method_5476()), true);
    return Command.SINGLE_SUCCESS;
  }

  private static void clear(ICurioStacksHandler stacksHandler) {

    for (int i = 0; i < stacksHandler.getSlots(); i++) {
      stacksHandler.getStacks().method_5447(i, class_1799.field_8037);
      stacksHandler.getCosmeticStacks().method_5447(i, class_1799.field_8037);
    }
  }
}
